/*
  very simple session management by pjj, v. 26.03.2012

  1) sessions are kept in *.txt files, located in /layouts dir
  2) list of sessions is kept in sessions.csv, located in /scripts dir
  3) sessions to restore are displayed from the newest to the oldest
  4) sessions to delete are displayed from the oldest to the newest

  known issues
  ============
  - limit of 5 sessions to be displayed (to restore or delete) is hardcoded
    (do we need more?)
  - file with session list (*.txt) are not deleted (can they be?)

  todo (maybe)
  ============
  - save current project (if it's opened) in sessions.csv, then open it
*/

var i, k; // simple counters
var sStr; // file with saved sessions read as a string
var sCounter = 0; // how many saved sessions are there
var sName[100]; // array with sessions' names
var currentDoc = MainApp.DocumentIndex;

// read file with saved sessions
if (MainApp.OpenFile("sessions.csv"))
{
  sStr = trim(Document.Text);
  TRJDocument.Close;
  MainApp.DocumentIndex = currentDoc;
}
// find sessions' names
for (i = 1; i <= length(sStr); i++)
{
  if ((Ord(sStr[i]) != 13) && (Ord(sStr[i]) != 10))
  {
    sName[sCounter] = sName[sCounter] + sStr[i];
  } else {
    if (Ord(sStr[i]) == 13)
    {
      sCounter++;
      // shouldn't be necessary, but in fact is:
      sName[sCounter] = "";
    }
  }
}
sCounter++;

function SaveThisSession()
{
  // save session's file list
  var sFileList = "";
  var fs = new TFileStream("../layouts/" + e.Text + ".txt", fmCreate);
  for (i = 0; i < MainApp.DocumentCount; i++)
  {
    MainApp.DocumentIndex = i;
    sFileList = sFileList + Document.FileName + "\n";
  }
  fs.Write(sFileList, length(sFileList));
  delete fs;

  // save session to sessions.csv
  if (length(sStr) == 0) // = empty file
  {
    sStr = e.Text;
  } else {
    sStr = sStr + "\n" + e.Text;
  }
  fs = new TFileStream("sessions.csv", fmCreate);
  fs.Write(sStr, length(sStr));
  delete fs;
  ShowMessage("session (hopefully) saved");
  MainApp.DocumentIndex = currentDoc;
  f.ModalResult = mrOk;
}

function RestoreThisSession(thisSessionId)
{
  // close all documents
  MainApp.CloseAll();
  // find session to restore
  var nr = Copy(thisSessionId.Name, 7, 1);
//  ShowMessage(nr);
  if (MainApp.OpenFile("../layouts/" + sName[nr] + ".txt"))
  {
    var files2Open = Document.Text;
    TRJDocument.Close;
  }
  var file = "";
  for (i = 1; i <= Length(files2Open); i++)
  {
    if ((files2Open[i] != "\n") && (files2Open[i] != "\r"))
    {
      file = file + files2Open[i];
    } else {
      if (file != "")
      {
        MainApp.OpenFile(file);
      }
      file = "";
    }
  }
  f.ModalResult = mrOk;
}

function DeleteThisSession(thisSessionId)
{
  var nr = Copy(thisSessionId.Name, 7, 1);
  // delete TXT file
  // NOTE is there a way to do it at all?
  // delete session from the list
  var newsStr = "";
  for (i = 0; i < 100; i++)
  {
    if ((i != nr) && (sName[i] != ""))
    {
      newsStr = newsStr + sName[i] + "\n";
    }
  }
  newsStr = trim(newsStr);
  // save new session list
  fs = new TFileStream("sessions.csv", fmCreate);
  fs.Write(newsStr, length(newsStr));
  delete fs;
  // re-read session list
  if (MainApp.OpenFile("sessions.csv"))
  {
    sStr = Document.Text;
    TRJDocument.Close;
    MainApp.DocumentIndex = currentDoc;
  }
  f.ModalResult = mrOk;
}

function mainButtonClick(Sender)
{
  if (Sender.Name == "btnSave")
  {
  // ---------------------------------------------------------------------------
  // -- saving session ---------------------------------------------------------
  // ---------------------------------------------------------------------------
    f.Caption = "Saving session...";
    // hide main buttons
    s.Visible = false;
    r.Visible = false;
    d.Visible = false;
    // show input box
    e = new TEdit(f);
    e.Parent = f;
    e.Text = "";
    e.Hint = "describe your session here";
    e.ShowHint = true;
    e.SetBounds(20, 20, 205, 20);
    e.AutoSelect = true;

    sb = new TButton(f);
    sb.Parent = f;
    sb.SetBounds(20, 60, 100, 25);
    sb.Caption = "save this session";
    sb.OnClick = &SaveThisSession;

  } else if (Sender.Name == "btnRestore") {
  // ---------------------------------------------------------------------------
  // -- restoring session ------------------------------------------------------
  // ---------------------------------------------------------------------------
    f.Caption = "Restoring session...";
    f.Height = 250;
    var rl = new TListBox(f);
    rl.Name = "restoreList";
    rl.Parent = f;
    rl.SetBounds(0, 0, 250, 250);
    // add "cancel" button
    rc = new TButton(f);
    rc.Parent = f;
    rc.Name = "btnRCancel";
    rc.SetBounds(80, 190, 95, 25);
    rc.Caption = "cancel";
    rc.OnClick = &mainButtonClick;

    // list sessions to restore
    k = sCounter - 1;
    var resbut;
    for (i = 0; i < 5; i++)
    {
      if (k >= 0)
      {
        resbut = new TButton(f);
        resbut.Name = "resbut" + IntToStr(k);
        resbut.Parent = f;
        resbut.SetBounds(15, 10 + i * 35, 215, 25);
        resbut.Caption = sName[k];
        resbut.OnClick = &RestoreThisSession;
      }
      k--;
    }
  } else if (Sender.Name == "btnDelete") {
  // ---------------------------------------------------------------------------
  // -- deleting session -------------------------------------------------------
  // ---------------------------------------------------------------------------
    f.Caption = "Deleting session...";
    f.Height = 250;
    var dl = new TListBox(f);
    dl.Name = "deleteSession";
    dl.Parent = f;
    dl.SetBounds(0, 0, 250, 250);

    // add "return" button
    bCancel = new TButton(f);
    bCancel.Parent = f;
    bCancel.Name = "btnCancelDel";
    bCancel.SetBounds(80, 190, 95, 25);
    bCancel.Caption = "cancel";
    bCancel.OnClick = &mainButtonClick;

    // list sessions to delete (from the oldest to the newest)
    k = 0;
    var delbut;
    for (i = 0; i < 5; i++)
    {
      if (sName[k] != "")
      {
        delbut = new TButton(f);
        delbut.Name = "delbut" + IntToStr(k);
        delbut.Parent = f;
        delbut.SetBounds(15, 10 + k * 35, 215, 25);
        delbut.Caption = sName[k];
        delbut.OnClick = &DeleteThisSession;
      }
      k++;
    }
  } else {
    f.ModalResult = mrOk;
  }
}

// show main app window
f = new TForm(nil);
f.Caption = "Simple session management";
f.BorderStyle = bsDialog;
f.Position = poScreenCenter;
f.Width = 250;
f.Height = 130;

l = new TListBox(f);
l.Parent = f;
l.SetBounds(0, 0, 250, 130);

s = new TButton(f);
s.Parent = f;
s.Name = "btnSave";
s.SetBounds(20, 15, 95, 25);
s.Caption = "save session";
if (MainApp.DocumentCount == 0)
{
  s.Enabled = false;
}
s.OnClick = &mainButtonClick;

r = new TButton(f);
r.Parent = f;
r.Name = "btnRestore";
r.SetBounds(130, 15, 95, 25);
r.Caption = "restore session";
if ((sCounter == 0) || (sName[0] == ""))
{
  r.Enabled = false;
}
r.OnClick = &mainButtonClick;

d = new TButton(f);
d.Parent = f;
d.Name = "btnDelete";
d.SetBounds(20, 60, 95, 25);
d.Caption = "delete session";
if ((sCounter == 0) || (sName[0] == ""))
{
  d.Enabled = false;
}
d.OnClick = &mainButtonClick;

c = new TButton(f);
c.Parent = f;
c.Name = "btnCancel";
c.SetBounds(130, 60, 95, 25);
c.Caption = "cancel";
c.OnClick = &mainButtonClick;

// show the main dialog
f.ShowModal;
f.Free;